@echo off


rem --------------------------------------------------------------------------------


set VERSION=1.0.0
set PROJECT_NAME=openbanking-main/openbanking-apps/openbanking-app-eventmanager
set DEPLOYABLE=openbanking-app-eventmanager-%VERSION%-ep-application.zip
set FRAGMENT_NAME=com.tibco.accelerators.openbanking.openbanking-fragment-eventmanager


rem Substitution variables
set APP_HOME=C:/workspace/OpenBanking
set STREAMBASE_PORT=10001
set WEB_SERVICES_PORT=80

rem set APP_ID
rem set MODEL_REPO
rem set AMS_HOST_NAME
rem set AMS_PORT
rem set AMS_USERNAME
rem set AMS_PASSWORD

set SUBSTITUTIONS=APP_HOME=%APP_HOME%,STREAMBASE_PORT=%STREAMBASE_PORT%,WEB_SERVICES_PORT=%WEB_SERVICES_PORT%

set NODE_NAME=A
set GROUP_NAME=sb
set CLUSTER_NAME=openbanking-sb


rem --------------------------------------------------------------------------------


rem Node configuration
set FULL_NODE_NAME=%NODE_NAME%.%GROUP_NAME%.%CLUSTER_NAME%
set ENGINE_NAME=default-engine-for-%FRAGMENT_NAME%
set LOG_PATH=../../../tmp/nodes/%FULL_NODE_NAME%/logs/%ENGINE_NAME%.log
set NODE_DIRECTORY=../../../tmp/nodes
set APP_ZIP=../../src/sb/%PROJECT_NAME%/target/%DEPLOYABLE%

rem StreamBase configuration
set STREAMBASE_HOME=C:\tibco\sb-cep\10.6

set CLASSPATH=%STREAMBASE_HOME%\lib\sbclient.jar;%CLASSPATH%
set PYTHONPATH=%PYTHONPATH%;%STREAMBASE_HOME%\lib\python2.7
set TIBCO_EP_HOME=%STREAMBASE_HOME%
set PATH=%STREAMBASE_HOME%\bin64;%STREAMBASE_HOME%\bin;%TIBCO_EP_HOME%\distrib\tibco\bin;%PATH%

if not defined JAVA_HOME set PATH=%STREAMBASE_HOME%\jdk\bin;%PATH%
if not defined JAVA_HOME set JAVA_HOME=%STREAMBASE_HOME%\jdk
if not defined TERR_HOME set TERR_HOME=%STREAMBASE_HOME%\terr


rem --------------------------------------------------------------------------------


rem Log SB version and start time
sbc --version
echo %time%

rem call command

call:%~1 2>NUL

IF ERRORLEVEL 1 (
	ECHO   Invalid argument: %1
	ECHO   Usage: %~n0 "install|start|restart|stop|remove|terminate|showlog"
	GOTO:end
)

exit /b


rem Install then Start node
:install
	if not exist "%APP_ZIP%" (
		echo [ERROR] Application deployable %APP_ZIP% not found. Build the project hierarchy using mvn clean install.
		goto:end
	)
	echo.
	echo Install node
	echo ^>^>^> epadmin install node application=%APP_ZIP% nodename=%FULL_NODE_NAME% nodedirectory=%NODE_DIRECTORY% substitutions="%SUBSTITUTIONS%"
	echo.
	epadmin install node application=%APP_ZIP% nodename=%FULL_NODE_NAME% nodedirectory=%NODE_DIRECTORY% substitutions="%SUBSTITUTIONS%"
	echo.
	echo Start node
	echo ^>^>^> epadmin servicename=%FULL_NODE_NAME% start node
	echo.
	epadmin servicename=%FULL_NODE_NAME% start node
	goto:showlog
	
rem Start node already installed
:start
	echo.
	echo Start node
	echo ^>^>^> epadmin servicename=%FULL_NODE_NAME% start node
	echo.
	epadmin servicename=%FULL_NODE_NAME% start node
	goto:showlog
	
rem Restart node and coordinator
:restart
	echo.
	echo Restart node coordinator and node
	echo ^>^>^> epadmin start node --installpath=%NODE_DIRECTORY%/%FULL_NODE_NAME%
	echo.
	epadmin start node --installpath=%NODE_DIRECTORY%/%FULL_NODE_NAME%
	goto:showlog

rem Stop node
:stop
	echo.
	echo Stop node
	echo ^>^>^> epadmin servicename=%FULL_NODE_NAME% stop node
	echo.
	epadmin servicename=%FULL_NODE_NAME% stop node
	goto:end
	
rem Stop and Remove node
:remove
	echo.
	echo Stop node
	echo ^>^>^> epadmin servicename=%FULL_NODE_NAME% stop node
	echo.
	epadmin servicename=%FULL_NODE_NAME% stop node
	echo.
	echo Remove node
	echo ^>^>^> epadmin servicename=%FULL_NODE_NAME% remove node
	echo.
	epadmin servicename=%FULL_NODE_NAME% remove node
	goto:end

rem Terminate node coordinator
:terminate
	echo.
	echo Terminate node coordinator
	echo ^>^>^> epadmin servicename=%FULL_NODE_NAME% terminate node
	echo.
	epadmin servicename=%FULL_NODE_NAME% terminate node
	goto:end
	
:showlog
	echo.
	echo Tailing %ENGINE_NAME%
	echo ^>^>^> epadmin servicename=%FULL_NODE_NAME% tail logging enginename=%ENGINE_NAME%
	epadmin servicename=%FULL_NODE_NAME% tail logging enginename=%ENGINE_NAME%
	goto:end
	
rem End of processing
:end
	echo Done


