/*
* Copyright © 2020. TIBCO Software Inc.
* This file is subject to the license terms contained
* in the license file that is distributed with this file.
*/

package com.tibco.accelerators.operators.container;

import java.beans.*;

import com.streambase.sb.operator.parameter.*;

/**
* A BeanInfo class controls what properties are exposed, add 
* metadata about properties (such as which properties are optional), and access 
* special types of properties that can't be automatically derived via reflection. 
* If a BeanInfo class is present, only the properties explicitly declared in
* this class will be exposed by StreamBase.
*/
public class ContainerOperatorBeanInfo extends SBSimpleBeanInfo {

	/*
	* The order of properties below determines the order they are displayed within
	* the StreamBase Studio property view. 
	*/
	public SBPropertyDescriptor[] getPropertyDescriptorsChecked() throws IntrospectionException {
		SBPropertyDescriptor[] p = {
				new JavaEnumPropertyDescriptor<ContainerOperator.OperationEnum>("operation", ContainerOperator.class,
						ContainerOperator.OperationEnum.class).displayName("Operation").description(""),
				new SBPropertyDescriptor("containerName", ContainerOperator.class).displayName("Container Name")
						.description(""),
				new JavaEnumPropertyDescriptor<ContainerOperator.DeliveryEnum>("delivery", ContainerOperator.class, 
						ContainerOperator.DeliveryEnum.class)
						.displayName("App Delivery").description(""),
				new SBPropertyDescriptor("containerAppFilename", ContainerOperator.class)
						.displayName("Container App Filename").description(""),
				new SBPropertyDescriptor("containerAppBlob", ContainerOperator.class)
						.displayName("Container App Blob").description(""),
				new SBPropertyDescriptor("containerOptions", ContainerOperator.class).displayName("Container Options")
						.description(""), };
		return p;
	}

}
