/*
* Copyright © 2020. TIBCO Software Inc.
* This file is subject to the license terms contained
* in the license file that is distributed with this file.
*/

package com.tibco.accelerators.operators.file;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;

import com.streambase.sb.*;
import com.streambase.sb.operator.*;
import com.tibco.accelerators.operators.file.handler.FileListHandler;

/**
 * Generated by JDT StreamBase Client Templates (Version: 7.6.7.1702081658).
 *
 * This class is used as a Java Operator in a StreamBase application.
 * One instance will be created for each Java Operator in a StreamBase 
 * application. 
 * <p>
 * Enqueue methods should only be called from processTuple.
 * <p>
 * If there is any state that the operator instance needs to maintain beyond the
 * lifetime of the instance, the setSessionState/getSessionState methods can
 * be used.
 * @see Parameterizable
 * @see Operator
 * For in-depth information on implementing a custom Java Operator, please see
 * "Developing StreamBase Java Operators" in the StreamBase documentation.
 */
public class FileListOperator extends Operator implements Parameterizable {

	public static final long serialVersionUID = 1490868795723L;
	private Logger logger;
	
	// Properties
	private String displayName = "File List";
	private Schema[] outputSchemas = null;
	
	// Adapter properties
	private String listPath;
	private boolean searchSubDirs;
	private String listFilter;
	private boolean oneTuplePerFile;
	private boolean synchronousList;
	private boolean passthroughFields;
	
	// Local variables
	private int inputPorts = 1;
	private int outputPorts = 1;

	/**
	 * The constructor is called when the Operator instance is created, but before the Operator 
	 * is connected to the StreamBase application. We recommended that you set the initial input
	 * port and output port count in the constructor by calling setPortHints(inPortCount, outPortCount).
	 * The default is 1 input port, 1 output port. The constructor may also set default values for 
	 * operator parameters. These values will be displayed in StreamBase Studio when a new instance
	 * of this operator is  dragged to the canvas, and serve as the default values for omitted
	 * optional parameters.
	 */
	public FileListOperator() {
		super();
		logger = getLogger();
		setPortHints(inputPorts, outputPorts);
		setDisplayName(displayName);
		setShortDisplayName(this.getClass().getSimpleName());
		
		setListPath("");
		setSearchSubDirs(false);
		setListFilter("");
		setOneTuplePerFile(false);
		setSynchronousList(true);
		setPassthroughFields(false);

	}

	/**
	 * The typecheck method is called after the Operator instance is connected in the StreamBase
	 * application, allowing the Operator to validate its properties. The Operator class may 
	 * change the number of input or output ports by calling the requireInputPortCount(portCount)
	 * method or the setOutputSchema(schema, portNum) method. If the verifyInputPortCount method 
	 * is passed a different number of ports than the Operator currently has, a PortMismatchException
	 * (subtype of TypecheckException) is thrown.
	 */
	public void typecheck() throws TypecheckException {
		// typecheck: require a specific number of input ports
		requireInputPortCount(inputPorts);
		setCaptureStrategy(CaptureTransformStrategy.NEST);

		// Get the input schema
		Schema inputSchema = getInputSchema(0);
		
		// Validate list path
		if(this.getListPath() == null || this.getListPath().length() == 0) {
			try {
				inputSchema.getField("listPath");
			} catch (TupleException e) {
				throw new PropertyTypecheckException("filename", "List path not specified in Adapter Properties or input tuple");
			}		
		}
		
		// Define output schemas
		Schema fileSchema = new Schema(null,
				Schema.createField(DataType.STRING, "absolutePath"),
				Schema.createField(DataType.STRING, "canonicalPath"),
				Schema.createField(DataType.STRING, "name"),
				Schema.createField(DataType.STRING, "parent"),
				Schema.createField(DataType.STRING, "path"),
				Schema.createField(DataType.TIMESTAMP, "lastModified"),
				Schema.createField(DataType.LONG, "length"),
				Schema.createField(DataType.STRING, "type"));					
		Schema dataSchema = null;
		if(this.getOneTuplePerFile() == true) {
			dataSchema = fileSchema;
		}
		else {
			dataSchema = new Schema(null,
				Schema.createListField("files", CompleteDataType.forTuple(fileSchema)));
		}
		
		
		// Check for passthrough
		if(this.getPassthroughFields() == true) {
			try {
				Schema passthroughSchema = inputSchema.getField("passthrough").getSchema();
				List<Schema.Field> fields = new ArrayList<Schema.Field>(dataSchema.fields());
				fields.add(Schema.createTupleField("passthrough", passthroughSchema));
				dataSchema = new Schema(null, fields); 
			} catch (TupleException e) {
				e.printStackTrace();
			}
		}

		// Set data output schema
		setOutputSchema(0, dataSchema);
		
	}

	/**
	 * This method will be called by the StreamBase server for each Tuple given
	 * to the Operator to process. This is the only time an operator should 
	 * enqueue output Tuples.
	 * @param inputPort the input port that the tuple is from (ports are zero based)
	 * @param tuple the tuple from the given input port
	 * @throws StreamBaseException Terminates the application.
	 */
	public void processTuple(int inputPort, Tuple tuple)
			throws StreamBaseException {
		FileListHandler operation = new FileListHandler(logger, this, tuple);
		if(this.getSynchronousList() == true) {
			operation.run();
		}
		else {
			operation.start();
		}
	}

	/**
	 * If typecheck succeeds, the init method is called before the StreamBase application
	 * is started. Note that your Operator class is not required to define the init method,
	 * unless (for example) you need to perform initialization of a resource such as a JDBC
	 * pool, if your operator is making JDBC calls. StreamBase Studio does not call this
	 * during authoring.
	 */
	public void init() throws StreamBaseException {
		super.init();
		// for best performance, consider caching input or output Schema.Field objects for
		// use later in processTuple()
		outputSchemas = new Schema[outputPorts];

		for (int i = 0; i < outputPorts; ++i) {
			outputSchemas[i] = getRuntimeOutputSchema(i);
		}
	}

	public Schema getCachedOutputSchema(int i) {
		return outputSchemas[i];
	}
	
	/**
	 *  The shutdown method is called when the StreamBase server is in the process of shutting down.
	 */
	public void shutdown() {

	}

	@Override
	public URL getIconResource(IconKind iconType) {
		switch (iconType) {
		case CANVAS_OVERLAY_ICON:
			return getClass().getResource("icons/filelist_48.png");
		case PALETTE_ICON_LARGE:
			return getClass().getResource("icons/filelist_32.png");
		case PALETTE_ICON_SMALL:
			return getClass().getResource("icons/filelist_16.png");
		}
		return null;
	}

	/***************************************************************************************
	 * The getter and setter methods provided by the Parameterizable object.               *
	 * StreamBase Studio uses them to determine the name and type of each property         *
	 * and obviously, to set and get the property values.                                  *
	 ***************************************************************************************/

	/**
	 * 
	 * @param listPath the list path
	 */
	public void setListPath(String listPath) {
		this.listPath = listPath;
	}

	/**
	 * 
	 * @return the list path
	 */
	public String getListPath() {
		return this.listPath;
	}

	/**
	 * 
	 * @param searchSubDirs search sub dirs
	 */
	public void setSearchSubDirs(boolean searchSubDirs) {
		this.searchSubDirs = searchSubDirs;
	}

	/**
	 * 
	 * @return search sub dirs
	 */
	public boolean getSearchSubDirs() {
		return this.searchSubDirs;
	}

	/**
	 * 
	 * @param listFilter the list filter
	 */	
	public void setListFilter(String listFilter) {
		this.listFilter = listFilter;
	}

	/**
	 * 
	 * @return the list filter
	 */
	public String getListFilter() {
		return this.listFilter;
	}

	/**
	 * 
	 * @param oneTuplePerFile one tuple per file
	 */
	public void setOneTuplePerFile(boolean oneTuplePerFile) {
		this.oneTuplePerFile = oneTuplePerFile;
	}

	/**
	 * 
	 * @return one tuple per file
	 */
	public boolean getOneTuplePerFile() {
		return this.oneTuplePerFile;
	}

	/**
	 * 
	 * @param synchronousList synchrnous list
	 */
	public void setSynchronousList(boolean synchronousList) {
		this.synchronousList = synchronousList;
	}

	/**
	 * 
	 * @return synchrnous list
	 */
	public boolean getSynchronousList() {
		return this.synchronousList;
	}

	/**
	 * 
	 * @param passthroughFields passthrough fields
	 */
	public void setPassthroughFields(boolean passthroughFields) {
		this.passthroughFields = passthroughFields;
	}

	/**
	 * 
	 * @return passthrough fields
	 */
	public boolean getPassthroughFields() {
		return this.passthroughFields;
	}

	/** For detailed information about shouldEnable methods, see interface Parameterizable java doc 
	 *  @see Parameterizable 
	 */

	/**
	 * 
	 * @return enable list path
	 */
	public boolean shouldEnableListPath() {
		return true;
	}

	/**
	 * 
	 * @return enable search sub dirs
	 */
	public boolean shouldEnableSearchSubDirs() {
		return true;
	}
	
	/**
	 * 
	 * @return enable list filter
	 */
	public boolean shouldEnableListFilter() {
		return true;
	}

	/**
	 * 
	 * @return enable one tuple per file
	 */
	public boolean shouldEnableOneTuplePerFile() {
		return true;
	}
	
	/**
	 * 
	 * @return enable synchrnous list
	 */
	public boolean shouldEnableSynchronousList() {
		return true;
	}
	
	/**
	 * 
	 * @return enable passthrough fields
	 */
	public boolean shouldEnablePassthroughFields() {
		return true;
	}

}
