/*
* Copyright © 2020. TIBCO Software Inc.
* This file is subject to the license terms contained
* in the license file that is distributed with this file.
*/

package com.tibco.accelerators.operators.file;

import java.net.URL;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;

import com.streambase.sb.*;
import com.streambase.sb.operator.*;
import com.tibco.accelerators.operators.file.handler.FileReaderHandler;
import com.tibco.accelerators.operators.file.parse.ParseHeader;
import com.tibco.accelerators.operators.file.parse.ParseIncompleteRecords;

/**
 * Generated by JDT StreamBase Client Templates (Version: 7.6.7.1702081658).
 *
 * This class is used as a Java Operator in a StreamBase application.
 * One instance will be created for each Java Operator in a StreamBase 
 * application. 
 * <p>
 * Enqueue methods should only be called from processTuple.
 * <p>
 * If there is any state that the operator instance needs to maintain beyond the
 * lifetime of the instance, the setSessionState/getSessionState methods can
 * be used.
 * @see Parameterizable
 * @see Operator
 * For in-depth information on implementing a custom Java Operator, please see
 * "Developing StreamBase Java Operators" in the StreamBase documentation.
 */
public class FileReaderOperator extends Operator implements Parameterizable {

	public static final long serialVersionUID = 1490695743979L;
	private Logger logger;

	// Properties
	private String displayName = "File Reader";
	private Schema[] outputSchemas = null;
	
	// Adapter Properties
	private String filename;
	private boolean readAsResource;
	private boolean useDefaultCharset;
	private String characterSet;
	private boolean readAsBinary;
	private boolean synchronousRead;
	private String parseAs;
	private boolean eventPort;
	private boolean passthroughFields;

	// Line Options
	private boolean oneTuplePerLine;
	private String lineSeparator;
	private String listTupleName;
	private String headerType;
	private ParseHeader headerTypeAction;
	
	// CSV Options
	private String fieldDelimiter;
	private boolean emptyAsNull;
	private String nullValue;
	private String timestampFormat;
	private boolean preserveWhitespace;
	private String incompleteRecords;
	private ParseIncompleteRecords incompleteRecordsAction;
	
	// Edit Schema
	private Schema schemaOut;

	// Local variables
	private int inputPorts = 1;
	private int outputPortsNoEvent = 1;
	private int outputPortsWithEvent = 2;
	
	
	//private int nextOutputPort = 0;
	//private Schema[] outputSchemas; // caches the Schemas given during init() for use at processTuple()

	/**
	 * The constructor is called when the Operator instance is created, but before the Operator 
	 * is connected to the StreamBase application. We recommended that you set the initial input
	 * port and output port count in the constructor by calling setPortHints(inPortCount, outPortCount).
	 * The default is 1 input port, 1 output port. The constructor may also set default values for 
	 * operator parameters. These values will be displayed in StreamBase Studio when a new instance
	 * of this operator is  dragged to the canvas, and serve as the default values for omitted
	 * optional parameters.
	 */
	public FileReaderOperator() {
		super();
		logger = getLogger();
		if(this.getEventPort() == true)
			setPortHints(inputPorts, outputPortsWithEvent);
		else 
			setPortHints(inputPorts, outputPortsNoEvent);
			
		setDisplayName(displayName);
		setShortDisplayName(this.getClass().getSimpleName());
		
		// Adapter Properties
		setFilename("");
		setUseDefaultCharset(true);
		setCharacterSet("");
		setReadAsBinary(false);
		setSynchronousRead(true);
		setParseAs("None");
		setEventPort(false);
		setPassthroughFields(false);

		// Line Options
		setOneTuplePerLine(true);
		setLineSeparator("");
		setListTupleName("");

		// CSV Options
		setFieldDelimiter(",");
		setEmptyAsNull(true);
		setNullValue("null");
		setTimestampFormat("yyyy-MM-dd HH:mm:ss.SSSZ");
		setPreserveWhitespace(false);
		setHeaderType("No header");
		setIncompleteRecords("Populate with nulls");
		
		// Edit Schema
		setSchemaOut(null);

	}

	/**
	 * The typecheck method is called after the Operator instance is connected in the StreamBase
	 * application, allowing the Operator to validate its properties. The Operator class may 
	 * change the number of input or output ports by calling the requireInputPortCount(portCount)
	 * method or the setOutputSchema(schema, portNum) method. If the verifyInputPortCount method 
	 * is passed a different number of ports than the Operator currently has, a PortMismatchException
	 * (subtype of TypecheckException) is thrown.
	 */
	public void typecheck() throws TypecheckException {
		// typecheck: require a specific number of input ports
		requireInputPortCount(inputPorts);
		setCaptureStrategy(CaptureTransformStrategy.NEST);

		// Get the input schema
		Schema inputSchema = getInputSchema(0);
		
		// Validate filename
		if(this.getFilename() == null || this.getFilename().length() == 0) {
			try {
				inputSchema.getField("filename");
			} catch (TupleException e) {
				throw new PropertyTypecheckException("filename", "Filename not specified in Adapter Properties or input filename tuple");
			}		
		}
		
		// Validate characterset
		if(this.getUseDefaultCharset() == false) {
			if(this.getCharacterSet() == null || this.getCharacterSet().length() == 0) {
				throw new PropertyTypecheckException("characterSet", "Character Set must be specified when not using default charset");
			}
			else {
				Charset charset = Charset.availableCharsets().get(this.getCharacterSet());
				if(charset == null) {
					throw new PropertyTypecheckException("characterSet", "Charset [" + this.getCharacterSet() + "] invalid or unsupported");
				}
			}
		}
		
		// Validate list tuple name
		if((this.getParseAs().equals("Line") || this.getParseAs().equals("CSV")) && 
				this.getOneTuplePerLine() == false && 
				(this.getListTupleName() == null || this.getListTupleName().length() == 0)) {
			throw new PropertyTypecheckException("listTupleName", "List tuple name must be specified where one tuple per line is not enabled");
		}
		
		// Define output schemas
		Schema dataSchema = null;
		
		// Port 0 is always data port
		if(this.getParseAs().equals("None")) {
			if(this.getReadAsBinary() == true) {
				dataSchema = new Schema(null,
						Schema.createField(DataType.STRING, "filename"),
						Schema.createField(DataType.BLOB, "content"));					
			}
			else {
				dataSchema = new Schema(null,
						Schema.createField(DataType.STRING, "filename"),
						Schema.createField(DataType.STRING, "content"));					
			}
		}
		else if(this.getParseAs().equals("Line")) {
			if(this.getOneTuplePerLine() == true) {
				dataSchema = new Schema(null,
						Schema.createField(DataType.STRING, "filename"),
						Schema.createField(DataType.STRING, "line"));
			}
			else {
				dataSchema = new Schema(null,
						Schema.createField(DataType.STRING, "filename"),
						Schema.createListField(this.getListTupleName(), CompleteDataType.forString()));					
			}
		}
		else if(this.getParseAs().equals("CSV")) {
			if(this.getOneTuplePerLine() == true) {
				dataSchema = this.getSchemaOut();
			}
			else {
				dataSchema = new Schema(null,
						Schema.createListField(this.getListTupleName(), CompleteDataType.forTuple(this.getSchemaOut())));					
			}
		}
		else if(this.getParseAs().equals("XML")) {
			dataSchema = new Schema(null,
					Schema.createField(DataType.STRING, "filename"),
					Schema.createField(DataType.STRING, "xmlContent"));					
		}
		else if(this.getParseAs().equals("JSON")) {
			dataSchema = this.getSchemaOut();
		}
		
		// Check for passthrough
		if(this.getPassthroughFields() == true && inputSchema.hasField("passthrough")) {
			try {
				Schema passthroughSchema = inputSchema.getField("passthrough").getSchema();
				List<Schema.Field> fields = new ArrayList<Schema.Field>(dataSchema.fields());
				fields.add(Schema.createTupleField("passthrough", passthroughSchema));
				dataSchema = new Schema(null, fields); 
			} catch (TupleException e) {
				e.printStackTrace();
			}
		}

		// Set data output schema
		setOutputSchema(0, dataSchema);
		
		// Port 1 is always event port, if enabled
		if(this.getEventPort() == true) {
			Schema eventSchema = new Schema(null, 
					Schema.createField(DataType.STRING, "type"),
					Schema.createField(DataType.STRING, "action"),
					Schema.createField(DataType.STRING, "object"),
					Schema.createField(DataType.STRING, "message"));
			setOutputSchema(1, eventSchema);
		}
				
	}

	/**
	 * This method will be called by the StreamBase server for each Tuple given
	 * to the Operator to process. This is the only time an operator should 
	 * enqueue output Tuples.
	 * @param inputPort the input port that the tuple is from (ports are zero based)
	 * @param tuple the tuple from the given input port
	 * @throws StreamBaseException Terminates the application.
	 */
	public void processTuple(int inputPort, Tuple tuple)
			throws StreamBaseException {

		FileReaderHandler operation = new FileReaderHandler(logger, this, tuple);
		if(this.getSynchronousRead() == true) {
			operation.run();
		}
		else {
			operation.start();
		}
		
	}

	/**
	 * If typecheck succeeds, the init method is called before the StreamBase application
	 * is started. Note that your Operator class is not required to define the init method,
	 * unless (for example) you need to perform initialization of a resource such as a JDBC
	 * pool, if your operator is making JDBC calls. StreamBase Studio does not call this
	 * during authoring.
	 */
	public void init() throws StreamBaseException {
		super.init();

		int outputPorts = 0;
		if(this.getEventPort() == true)
			outputPorts = outputPortsWithEvent;
		else 
			outputPorts = outputPortsNoEvent;
		
		outputSchemas = new Schema[outputPorts];

		for (int i = 0; i < outputPorts; ++i) {
			outputSchemas[i] = getRuntimeOutputSchema(i);
		}
	}

	public Schema getCachedOutputSchema(int i) {
		return outputSchemas[i];
	}
	
	/**
	 *  The shutdown method is called when the StreamBase server is in the process of shutting down.
	 */
	public void shutdown() {

	}

	@Override
	public URL getIconResource(IconKind iconType) {
		switch (iconType) {
		case CANVAS_OVERLAY_ICON:
			return getClass().getResource("icons/filereader_48.png");
		case PALETTE_ICON_LARGE:
			return getClass().getResource("icons/filereader_32.png");
		case PALETTE_ICON_SMALL:
			return getClass().getResource("icons/filereader_16.png");
		}
		return null;
	}
	
	/***************************************************************************************
	 * The getter and setter methods provided by the Parameterizable object.               *
	 * StreamBase Studio uses them to determine the name and type of each property         *
	 * and obviously, to set and get the property values.                                  *
	 ***************************************************************************************/
	// Adapter Properties
	/**
	 * 
	 * @param filename the filename
	 */
	public void setFilename(String filename) {
		this.filename = filename;
	}

	/**
	 * 
	 * @return the filename
	 */
	public String getFilename() {
		return this.filename;
	}

	/**
	 * 
	 * @param readAsResource the readAsResource
	 */
	public void setReadAsResource(boolean readAsResource) {
		this.readAsResource = readAsResource;
	}

	/**
	 * 
	 * @return the readAsResource
	 */
	public boolean getReadAsResource() {
		return this.readAsResource;
	}

	/**
	 * 
	 * @param useDefaultCharset use default charset
	 */
	public void setUseDefaultCharset(boolean useDefaultCharset) {
		this.useDefaultCharset = useDefaultCharset;
	}

	/**
	 * 
	 * @return use default charset
	 */
	public boolean getUseDefaultCharset() {
		return this.useDefaultCharset;
	}

	/**
	 * 
	 * @param characterSet the character set
	 */
	public void setCharacterSet(String characterSet) {
		this.characterSet = characterSet;
	}

	/**
	 * 
	 * @return the character set
	 */
	public String getCharacterSet() {
		return this.characterSet;
	}

	/**
	 * 
	 * @param readAsBinary read as binary
	 */
	public void setReadAsBinary(boolean readAsBinary) {
		this.readAsBinary = readAsBinary;
	}

	/**
	 * 
	 * @return read as binary
	 */
	public boolean getReadAsBinary() {
		return this.readAsBinary;
	}

	/**
	 * 
	 * @param synchronousRead synchronous read
	 */
	public void setSynchronousRead(boolean synchronousRead) {
		this.synchronousRead = synchronousRead;
	}

	/**
	 * 
	 * @return synchronous read
	 */
	public boolean getSynchronousRead() {
		return this.synchronousRead;
	}

	/**
	 * 
	 * @param passthroughFields passthrough fields
	 */
	public void setPassthroughFields(boolean passthroughFields) {
		this.passthroughFields = passthroughFields;
	}

	/**
	 * 
	 * @return passthrough fields
	 */
	public boolean getPassthroughFields() {
		return this.passthroughFields;
	}

	/**
	 * 
	 * @param eventPort event port
	 */
	public void setEventPort(boolean eventPort) {
		this.eventPort = eventPort;
		if(this.getEventPort() == true)
			setPortHints(inputPorts, outputPortsWithEvent);
		else 
			setPortHints(inputPorts, outputPortsNoEvent);
	}

	/**
	 *  
	 * @return event port
	 */
	public boolean getEventPort() {
		return this.eventPort;
	}

	/**
	 * 
	 * @param parseAs parse as
	 */
	public void setParseAs(String parseAs) {
		this.parseAs = parseAs;
	}

	/**
	 * 
	 * @return parse as
	 */
	public String getParseAs() {
		return this.parseAs;
	}

	// Line Options
	/**
	 * 
	 * @param oneTuplePerLine one tuple per line
	 */
	public void setOneTuplePerLine(boolean oneTuplePerLine) {
		this.oneTuplePerLine = oneTuplePerLine;
	}

	/**
	 * 
	 * @return parse as
	 */
	public boolean getOneTuplePerLine() {
		return this.oneTuplePerLine;
	}

	/**
	 * 
	 * @param lineSeparator line separator
	 */
	public void setLineSeparator(String lineSeparator) {
		this.lineSeparator = lineSeparator;
	}

	/**
	 * 
	 * @return line separator
	 */
	public String getLineSeparator() {
		return this.lineSeparator;
	}

	/**
	 * 
	 * @param listTupleName list tuple name
	 */
	public void setListTupleName(String listTupleName) {
		this.listTupleName = listTupleName;
	}

	/**
	 * 
	 * @return list tuple name
	 */
	public String getListTupleName() {
		return this.listTupleName;
	}

	// CSV Options
	/**
	 * 
	 * @param fieldDelimiter field delimiter
	 */
	public void setFieldDelimiter(String fieldDelimiter) {
		this.fieldDelimiter = fieldDelimiter;
	}

	/**
	 * 
	 * @return field delimiter
	 */
	public String getFieldDelimiter() {
		return this.fieldDelimiter;
	}

	/**
	 * 
	 * @param emptyAsNull empty as null
	 */
	public void setEmptyAsNull(boolean emptyAsNull) {
		this.emptyAsNull = emptyAsNull;
	}

	/**
	 * 
	 * @return empty as null
	 */
	public boolean getEmptyAsNull() {
		return this.emptyAsNull;
	}

	/**
	 * 
	 * @param nullValue null value
	 */
	public void setNullValue(String nullValue) {
		this.nullValue = nullValue;
	}
	
	/**
	 * 
	 * @return null value
	 */
	public String getNullValue() {
		return this.nullValue;
	}
	
	/**
	 * 
	 * @param timestampFormat timestamp format
	 */
	public void setTimestampFormat(String timestampFormat) {
		this.timestampFormat = timestampFormat;
	}

	/**
	 * 
	 * @return timestamp format
	 */
	public String getTimestampFormat() {
		return this.timestampFormat;
	}

	/**
	 * 
	 * @param preserveWhitespace preserve whitespace
	 */
	public void setPreserveWhitespace(boolean preserveWhitespace) {
		this.preserveWhitespace = preserveWhitespace;
	}

	/**
	 * 
	 * @return preserve whitespace
	 */
	public boolean getPreserveWhitespace() {
		return this.preserveWhitespace;
	}

	/**
	 * 
	 * @param headerType header type
	 */
	public void setHeaderType(String headerType) {
		this.headerType = headerType;
		if(this.headerType.equals("No header"))
			this.headerTypeAction = ParseHeader.NO_HEADER;
		else if(this.headerType.equals("Ignore header"))
			this.headerTypeAction = ParseHeader.IGNORE_HEADER;
		else if(this.headerType.equals("Read header"))
			this.headerTypeAction = ParseHeader.READ_HEADER;
	}

	/**
	 * 
	 * @return header type
	 */
	public String getHeaderType() {
		return this.headerType;
	}

	/**
	 * 
	 * @return header type action
	 */
	public ParseHeader getHeaderTypeAction() {
		return this.headerTypeAction;
	}
	
	/**
	 * 
	 * @param incompleteRecords incomplete records
	 */
	public void setIncompleteRecords(String incompleteRecords) {
		this.incompleteRecords = incompleteRecords;
		if(this.incompleteRecords.equals("Discard"))
			this.incompleteRecordsAction = ParseIncompleteRecords.DISCARD;
		else if(this.incompleteRecords.equals("Populate with nulls"))
			this.incompleteRecordsAction = ParseIncompleteRecords.POPULATE_WITH_NULLS;
	}

	/**
	 * 
	 * @return incomplete records
	 */
	public String getIncompleteRecords() {
		return this.incompleteRecords;
	}

	/**
	 * 
	 * @return incomplete records action
	 */
	public ParseIncompleteRecords getIncompleteRecordsAction() {
		return this.incompleteRecordsAction;
	}
	
	/**
	 * 
	 * @param schemaOut schema out
	 */
	// Edit Schema
	public void setSchemaOut(Schema schemaOut) {
		this.schemaOut = schemaOut;
	}

	/**
	 * 
	 * @return schema out
	 */
	public Schema getSchemaOut() {
		return this.schemaOut;
	}

	/** For detailed information about shouldEnable methods, see interface Parameterizable java doc 
	 *  @see Parameterizable 
	 */

	// Adapter Properties
	/**
	 * 
	 * @return enable filename
	 */
	public boolean shouldEnableFilename() {
		return true;
	}

	/**
	 * 
	 * @return enable readAsResource
	 */
	public boolean shouldEnableReadAsResource() {
		return true;
	}

	/**
	 * 
	 * @return enable use default charset
	 */
	public boolean shouldEnableUseDefaultCharset() {
		if(this.getReadAsBinary() == false)
			return true;
		return false;
	}

	/**
	 * 
	 * @return enable character set
	 */
	public boolean shouldEnableCharacterSet() {
		if(this.getUseDefaultCharset() == false && this.getReadAsBinary() == false)
			return true;
		return false;
	}

	/**
	 * 
	 * @return enable read as binary
	 */
	public boolean shouldEnableReadAsBinary() {
		if(this.getParseAs().equals("None"))
			return true;
		return false;
	}

	/**
	 * 
	 * @return enable passthrough fields
	 */
	public boolean shouldEnablePassthroughFields() {
		return true;
	}

	/**
	 * 
	 * @return enable event port
	 */
	public boolean shouldEnableEventPort() {
		return true;
	}

	/**
	 * 
	 * @return enable parse as
	 */
	public boolean shouldEnableParseAs() {
		if(this.getReadAsBinary() == true)
			return false;
		return true;
	}

	// Line Options
	/**
	 * 
	 * @return enable one tuple per line
	 */
	public boolean shouldEnableOneTuplePerLine() {
		if(this.getParseAs().equals("Line") || this.getParseAs().equals("CSV"))
			return true;
		return false;
	}

	/**
	 * 
	 * @return enable line separator
	 */
	public boolean shouldEnableLineSeparator() {
		if((this.getParseAs().equals("Line") || this.getParseAs().equals("CSV")))
			return true;
		return false;
	}

	/**
	 * 
	 * @return enable list tuple name
	 */
	public boolean shouldEnableListTupleName() {
		if((this.getParseAs().equals("Line") || this.getParseAs().equals("CSV")) && this.getOneTuplePerLine() == false)
			return true;
		return false;
	}

	/**
	 * 
	 * @return enable header type
	 */
	public boolean shouldEnableHeaderType() {
		if(this.getParseAs().equals("Line") || this.getParseAs().equals("CSV"))
			return true;
		return false;
	}

	// CSV Options
	/**
	 * 
	 * @return enable field delimiter
	 */
	public boolean shouldEnableFieldDelimiter() {
		if(this.getParseAs().equals("CSV"))
			return true;
		return false;
	}

	/**
	 * 
	 * @return enable empty as null
	 */
	public boolean shouldEnableEmptyAsNull() {
		if(this.getParseAs().equals("CSV"))
			return true;
		return false;
	}

	/**
	 * 
	 * @return enable null value
	 */
	public boolean shouldEnableNullValue() {
		if(this.getParseAs().equals("CSV") && this.getEmptyAsNull() == false)
			return true;
		return false;
	}

	/**
	 * 
	 * @return enable timestamp format
	 */
	public boolean shouldEnableTimestampFormat() {
		if(this.getParseAs().equals("CSV"))
			return true;
		return false;
	}

	/**
	 * 
	 * @return enable preserve whitespace
	 */
	public boolean shouldEnablePreserveWhitespace() {
		if(this.getParseAs().equals("CSV"))
			return true;
		return false;
	}

	/**
	 * 
	 * @return enable incomplete records
	 */
	public boolean shouldEnableIncompleteRecords() {
		if(this.getParseAs().equals("CSV"))
			return true;
		return false;
	}

	// Edit Schema
	/**
	 * 
	 * @return enable schema out
	 */
	public boolean shouldEnableSchemaOut() {
		if(this.getParseAs().equals("None"))
			return false;
		return true;
	}
}
