/*
* Copyright © 2020. TIBCO Software Inc.
* This file is subject to the license terms contained
* in the license file that is distributed with this file.
*/

package com.tibco.accelerators.operators.file;

import java.beans.*;

import com.streambase.sb.operator.parameter.*;

/**
 * A BeanInfo class controls what properties are exposed, add metadata about
 * properties (such as which properties are optional), and access special types
 * of properties that can't be automatically derived via reflection. If a
 * BeanInfo class is present, only the properties explicitly declared in this
 * class will be exposed by StreamBase.
 */
public class FileReaderOperatorBeanInfo extends SBSimpleBeanInfo {

	/*
	 * The order of properties below determines the order they are displayed
	 * within the StreamBase Studio property view.
	 */
	public SBPropertyDescriptor[] getPropertyDescriptorsChecked()
			throws IntrospectionException {
		SBPropertyDescriptor[] p = {
				// Adapter Properties
				new SBPropertyDescriptor("filename", FileReaderOperator.class)
						.displayName("File Name")
						.description("")
						.optional()
						.setUIHints(
								UIHints.create().setTab("Adapter Properties")),
				new SBPropertyDescriptor("readAsResource", FileReaderOperator.class)
						.displayName("Read As Resource")
						.description("")
						.optional()
						.setUIHints(
								UIHints.create().setTab("Adapter Properties")),
				new SBPropertyDescriptor("useDefaultCharset", FileReaderOperator.class)
						.displayName("Use Default Charset")
						.description("")
						.optional()
						.setUIHints(
								UIHints.create().setTab("Adapter Properties")),
				new SBPropertyDescriptor("characterSet", FileReaderOperator.class)
						.displayName("Character Set")
						.description("")
						.optional()
						.setUIHints(
								UIHints.create().setTab("Adapter Properties")),
				new SBPropertyDescriptor("readAsBinary", FileReaderOperator.class)
						.displayName("Read As Binary")
						.description("")
						.optional()
						.setUIHints(
								UIHints.create().setTab("Adapter Properties")),
				new SBPropertyDescriptor("synchronousRead", FileReaderOperator.class)
						.displayName("Synchronous Read")
						.description("")
						.optional()
						.setUIHints(
								UIHints.create().setTab("Adapter Properties")),
				new EnumPropertyDescriptor("parseAs", FileReaderOperator.class,
						new String[] { "None", "Line", "CSV", "JSON", "XML" })
						.displayName("Parse As")
						.description("")
						.optional()
						.setUIHints(
								UIHints.create().setTab("Adapter Properties")),
				new SBPropertyDescriptor("eventPort", FileReaderOperator.class)
						.displayName("Event Port")
						.description("")
						.optional()
						.setUIHints(
								UIHints.create().setTab("Adapter Properties")),
				new SBPropertyDescriptor("passthroughFields", FileReaderOperator.class)
						.displayName("Passthrough Fields")
						.description("")
						.optional()
						.setUIHints(
								UIHints.create().setTab("Adapter Properties")),

				// Line parsing options
				new SBPropertyDescriptor("oneTuplePerLine", FileReaderOperator.class)
						.displayName("One Tuple Per Line").description("")
						.optional()
						.setUIHints(UIHints.create().setTab("Line Options")),
				new SBPropertyDescriptor("lineSeparator", FileReaderOperator.class)
						.displayName("Line Separator").description("")
						.optional()
						.setUIHints(UIHints.create().setTab("Line Options")),
				new SBPropertyDescriptor("listTupleName", FileReaderOperator.class)
						.displayName("List Tuple Name").description("")
						.optional()
						.setUIHints(UIHints.create().setTab("Line Options")),
				new EnumPropertyDescriptor("headerType", FileReaderOperator.class,
						new String[] { "No header", "Ignore header",
								"Read header" }).displayName("Header Type")
						.description("").optional()
						.setUIHints(UIHints.create().setTab("Line Options")),

				// CSV Parsing Options
				new SBPropertyDescriptor("fieldDelimiter", FileReaderOperator.class)
						.displayName("Field Delimiter").description("")
						.optional()
						.setUIHints(UIHints.create().setTab("CSV Options")),
				new SBPropertyDescriptor("emptyAsNull", FileReaderOperator.class)
						.displayName("Empty As Null").description("")
						.optional()
						.setUIHints(UIHints.create().setTab("CSV Options")),
				new SBPropertyDescriptor("nullValue", FileReaderOperator.class)
						.displayName("Null Value").description("").optional()
						.setUIHints(UIHints.create().setTab("CSV Options")),
				new SBPropertyDescriptor("timestampFormat", FileReaderOperator.class)
						.displayName("Timestamp Format").description("")
						.optional()
						.setUIHints(UIHints.create().setTab("CSV Options")),
				new SBPropertyDescriptor("preserveWhitespace", FileReaderOperator.class)
						.displayName("Preserve Whitespace").description("")
						.optional()
						.setUIHints(UIHints.create().setTab("CSV Options")),
				new EnumPropertyDescriptor("incompleteRecords",
						FileReaderOperator.class, new String[] { "Discard",
								"Populate with nulls" })
						.displayName("Incomplete Records").description("")
						.optional()
						.setUIHints(UIHints.create().setTab("CSV Options")),

				// Edit schema
				new SBPropertyDescriptor("schemaOut", FileReaderOperator.class)
						.displayName("Schema Out").description("").optional()
						.setUIHints(UIHints.create().setTab("Edit Schema")), };
		return p;
	}

}
