/*
* Copyright © 2020. TIBCO Software Inc.
* This file is subject to the license terms contained
* in the license file that is distributed with this file.
*/

package com.tibco.accelerators.operators.file;

import java.net.URL;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;

import com.streambase.sb.*;
import com.streambase.sb.operator.*;
import com.tibco.accelerators.operators.file.handler.FileDoesntExist;
import com.tibco.accelerators.operators.file.handler.FileExists;
import com.tibco.accelerators.operators.file.handler.FileWriterHandler;

/**
 * Generated by JDT StreamBase Client Templates (Version: 7.6.7.1702081658).
 *
 * This class is used as a Java Operator in a StreamBase application. One
 * instance will be created for each Java Operator in a StreamBase application.
 * <p>
 * Enqueue methods should only be called from processTuple.
 * <p>
 * If there is any state that the operator instance needs to maintain beyond the
 * lifetime of the instance, the setSessionState/getSessionState methods can be
 * used.
 * 
 * @see Parameterizable
 * @see Operator For in-depth information on implementing a custom Java
 *      Operator, please see "Developing StreamBase Java Operators" in the
 *      StreamBase documentation.
 */
public class FileWriterOperator extends Operator implements Parameterizable {

	public static final long serialVersionUID = 1490695743979L;
	private Logger logger;

	// Properties
	private String displayName = "File Writer";
	private Schema[] outputSchemas = null;

	private Schema inputObjectSchema = null;
	
	// Adapter Properties
	private String filename;
	private String controlFilename;
	private String fileDoesntExist;
	private FileDoesntExist fileDoesntExistAction;
	private String fileExists;
	private FileExists fileExistsAction;
	private boolean useDefaultCharset;
	private String characterSet;
	private boolean writeAsBinary;
	private boolean synchronousWrite;
	private String renderAs;
	private boolean controlPort;
	private boolean eventPort;
	private boolean passthroughFields;

	// Line Options
	private boolean oneLinePerTuple;
	private String lineSeparator;
	private String listTupleName;

	// CSV Options
	private boolean includeHeader;
	private String fieldDelimiter;
	private boolean nullAsEmpty;
	private String nullValue;
	private String timestampFormat;

	// Local variables
	private int inputPortsNoControl = 1;
	private int inputPortsWithControl = 2;
	private int outputPortsNoEvent = 1;
	private int outputPortsWithEvent = 2;

	// private int nextOutputPort = 0;
	// private Schema[] outputSchemas; // caches the Schemas given during init()
	// for use at processTuple()

	/**
	 * The constructor is called when the Operator instance is created, but
	 * before the Operator is connected to the StreamBase application. We
	 * recommended that you set the initial input port and output port count in
	 * the constructor by calling setPortHints(inPortCount, outPortCount). The
	 * default is 1 input port, 1 output port. The constructor may also set
	 * default values for operator parameters. These values will be displayed in
	 * StreamBase Studio when a new instance of this operator is dragged to the
	 * canvas, and serve as the default values for omitted optional parameters.
	 */
	public FileWriterOperator() {
		super();
		logger = getLogger();
		int inports = inputPortsNoControl;
		if(this.getControlPort() == true)
			inports = inputPortsWithControl;
		int outports = outputPortsNoEvent;
		if(this.getEventPort() == true)
			outports = outputPortsWithEvent;
		setPortHints(inports, outports);

		setDisplayName(displayName);
		setShortDisplayName(this.getClass().getSimpleName());

		// Adapter Properties
		setFilename("");
		setFileDoesntExist("Create new file");
		setFileExists("Append to existing file");
		setUseDefaultCharset(true);
		setCharacterSet("");
		setSynchronousWrite(true);
		setRenderAs("None");
		setControlPort(false);
		setEventPort(false);
		setPassthroughFields(false);

		// Line Options
		setOneLinePerTuple(true);
		setLineSeparator("");
		setListTupleName("");

		// CSV Options
		setIncludeHeader(true);
		setFieldDelimiter(",");
		setNullAsEmpty(true);
		setNullValue("null");
		setTimestampFormat("yyyy-MM-dd HH:mm:ss.SSSZ");

	}

	/**
	 * The typecheck method is called after the Operator instance is connected
	 * in the StreamBase application, allowing the Operator to validate its
	 * properties. The Operator class may change the number of input or output
	 * ports by calling the requireInputPortCount(portCount) method or the
	 * setOutputSchema(schema, portNum) method. If the verifyInputPortCount
	 * method is passed a different number of ports than the Operator currently
	 * has, a PortMismatchException (subtype of TypecheckException) is thrown.
	 */
	public void typecheck() throws TypecheckException {
		// typecheck: require a specific number of input ports
		int inports = inputPortsNoControl;
		if (this.getControlPort() == true)
			inports = inputPortsWithControl;
		requireInputPortCount(inports);
		setCaptureStrategy(CaptureTransformStrategy.NEST);

		// Get the input schema
		Schema inputControlSchema = null;
		Schema inputEventSchema = null;
		if (this.getControlPort() == true) {
			inputControlSchema = getInputSchema(0);
			inputEventSchema = getInputSchema(1);
		} else
			inputEventSchema = getInputSchema(0);

		// Validate filename
		if (this.getFilename() == null || this.getFilename().length() == 0) {
			try {
				if (inputControlSchema != null)
					inputControlSchema.getField("filename");
				else
					throw new PropertyTypecheckException("filename",
							"Filename not specified in Adapter Properties or control input filename tuple");
			} catch (TupleException e) {
				throw new PropertyTypecheckException("filename",
						"Filename not specified in Adapter Properties or control input filename tuple");
			}
		}

		// Validate characterset
		if (this.getUseDefaultCharset() == false) {
			if (this.getCharacterSet() == null
					|| this.getCharacterSet().length() == 0) {
				throw new PropertyTypecheckException("characterSet",
						"Character Set must be specified when not using default charset");
			} else {
				Charset charset = Charset.availableCharsets().get(
						this.getCharacterSet());
				if (charset == null) {
					throw new PropertyTypecheckException("characterSet",
							"Charset [" + this.getCharacterSet()
									+ "] invalid or unsupported");
				}
			}
		}

		// Validate list tuple name
		if ((this.getRenderAs().equals("Line") || this.getRenderAs().equals("CSV"))
				&& this.getOneLinePerTuple() == false && (this.getListTupleName() == null || this.getListTupleName()
						.length() == 0)) {
			throw new PropertyTypecheckException("listTupleName",
					"List tuple name must be specified where one tuple per line is not enabled");
		}

		// Validate the content for Write as Binary
		if (this.getWriteAsBinary() == true) {
			if (inputEventSchema.getFieldCount() != 1) {
				throw new PropertyTypecheckException("writeAsBinary",
						"Input event tuple must be a single field of type block for binary render type");
			} else if (inputEventSchema.getField(0).getDataType() != DataType.BLOB) {
				throw new PropertyTypecheckException("writeAsBinary",
						"Input event tuple must be a single field of type block for binary render type");
			}
		}
		// Validate for non-binary
		else {
			// Validate the content for None type
			if (this.getRenderAs().equals("None")) {
				if (inputEventSchema.getFieldCount() != 1) {
					throw new PropertyTypecheckException("renderAs",
							"Input event tuple must be a single field of type string for None render type");
				} else if (inputEventSchema.getField(0).getDataType() != DataType.STRING) {
					throw new PropertyTypecheckException("renderAs",
							"Input event tuple must be a single field of type string for None render type");
				}
			}
	
			// Validate the content for Line type
			if (this.getRenderAs().equals("Line")) {
				if(this.getOneLinePerTuple() == true) {
					if (inputEventSchema.getFieldCount() != 1)
						throw new PropertyTypecheckException("renderAs", "Input event tuple must be a single field for Line render type with one line per tuple");
					else if (inputEventSchema.getField(0).getDataType() != DataType.STRING)
						throw new PropertyTypecheckException("renderAs", "Input event tuple must be a field of type string for Line render type with one line per tuple");
				}
				else {
					try {
						Schema.Field listTupleField = inputEventSchema.getField(this.getListTupleName());
						if (listTupleField.getDataType() != DataType.LIST)
							throw new PropertyTypecheckException("listTupleName", "Input event tuple list tuple field is not a list for Line render type with multiple lines per tuple");
						else if (listTupleField.getElementType() != CompleteDataType.forString())
							throw new PropertyTypecheckException("listTupleName", "Input event tuple list tuple field is not a list of strings for Line render type with multiple lines per tuple");
					} catch (TupleException ex) {
						throw new PropertyTypecheckException("listTupleName", "Input event tuple is missing list tuple field for Line render type with multiple lines per tuple");
					}
				}
			}
			
			// Validate the content for CSV type
			if (this.getRenderAs().equals("CSV")) {
				if(this.getOneLinePerTuple() == true) {
					if(inputEventSchema.getFieldCount() == 0)
						throw new PropertyTypecheckException("renderAs", "Input event tuple must have at least one field for CSV render type with one line per tuple");
					else {
						List<Schema.Field> fields = new ArrayList<Schema.Field>();
						for(int idx = 0; idx < inputEventSchema.getFieldCount(); idx++) {
							Schema.Field f = inputEventSchema.getField(idx);
							if(this.getPassthroughFields() == true && f.getName().equals("passthrough"))
								continue;
							fields.add(f);
						}
						this.inputObjectSchema = new Schema(null, fields);
					}
				}
				else {
					try {
						Schema.Field listTupleField = inputEventSchema.getField(this.getListTupleName());
						if (listTupleField.getDataType() != DataType.LIST)
							throw new PropertyTypecheckException("listTupleName", "Input event tuple list tuple field is not a list for CSV render type with multiple lines per tuple");
						else if (!(listTupleField.getElementType() instanceof CompleteDataType.TupleType))
							throw new PropertyTypecheckException("listTupleName", "Input event tuple list tuple field is not a list of tuples for CSVrender type with multiple lines per tuple");
					} catch (TupleException ex) {
						throw new PropertyTypecheckException("listTupleName", "Input event tuple is missing list tuple field for CSV render type with multiple lines per tuple");
					}
				}
	
				if(this.getFieldDelimiter() != null && this.getFieldDelimiter().length() > 1) {
					throw new PropertyTypecheckException("fieldDelimiter", "Field delimiter must be a single character");
				}
			}
	
			// Validate the content for JSON type
			if (this.getRenderAs().equals("JSON")) {
				List<Schema.Field> fields = new ArrayList<Schema.Field>();
				for(int idx = 0; idx < inputEventSchema.getFieldCount(); idx++) {
					Schema.Field f = inputEventSchema.getField(idx);
					if(this.getPassthroughFields() == true && f.getName().equals("passthrough"))
						continue;
					fields.add(f);
				}
				this.inputObjectSchema = new Schema(null, fields);
			}
		}
		
		// Define output schemas
		Schema dataSchema = new Schema(null, Schema.createField(
				DataType.STRING, "filename"));

		// Check for passthrough
		if (this.getPassthroughFields() == true) {
			try {
				Schema passthroughSchema = inputEventSchema.getField(
						"passthrough").getSchema();
				if(passthroughSchema != null) {
					List<Schema.Field> fields = new ArrayList<Schema.Field>(
							dataSchema.fields());
					fields.add(Schema.createTupleField("passthrough",
							passthroughSchema));
					dataSchema = new Schema(null, fields);
				}
			} catch (TupleException e) {
				e.printStackTrace();
			}
		}

		// Set data output schema
		setOutputSchema(0, dataSchema);

		// Port 1 is always event port, if enabled
		if (this.getEventPort() == true) {
			Schema eventSchema = new Schema(null, Schema.createField(
					DataType.STRING, "type"), Schema.createField(
					DataType.STRING, "action"), Schema.createField(
					DataType.STRING, "object"), Schema.createField(
					DataType.STRING, "message"));
			setOutputSchema(1, eventSchema);
		}

	}

	/**
	 * This method will be called by the StreamBase server for each Tuple given
	 * to the Operator to process. This is the only time an operator should
	 * enqueue output Tuples.
	 * 
	 * @param inputPort
	 *            the input port that the tuple is from (ports are zero based)
	 * @param tuple
	 *            the tuple from the given input port
	 * @throws StreamBaseException
	 *             Terminates the application.
	 */
	public void processTuple(int inputPort, Tuple tuple)
			throws StreamBaseException {

		if (this.getControlPort() == true && inputPort == 0) {
			try {
				this.controlFilename = tuple.getString("filename");
			}
			catch(Exception ex) {
				this.logger.warn(ex.getMessage());
			}
		}
		else {
			FileWriterHandler operation = new FileWriterHandler(logger, this,
					tuple);
			if (this.getSynchronousWrite() == true) {
				operation.run();
			} 
			else {
				operation.start();
			}
		}

	}

	/**
	 * If typecheck succeeds, the init method is called before the StreamBase
	 * application is started. Note that your Operator class is not required to
	 * define the init method, unless (for example) you need to perform
	 * initialization of a resource such as a JDBC pool, if your operator is
	 * making JDBC calls. StreamBase Studio does not call this during authoring.
	 */
	public void init() throws StreamBaseException {
		super.init();

		int outputPorts = 0;
		if (this.getEventPort() == true)
			outputPorts = outputPortsWithEvent;
		else
			outputPorts = outputPortsNoEvent;

		outputSchemas = new Schema[outputPorts];

		for (int i = 0; i < outputPorts; ++i) {
			outputSchemas[i] = getRuntimeOutputSchema(i);
		}
	}

	public Schema getCachedOutputSchema(int i) {
		return outputSchemas[i];
	}

	/**
	 * The shutdown method is called when the StreamBase server is in the
	 * process of shutting down.
	 */
	public void shutdown() {

	}

	@Override
	public URL getIconResource(IconKind iconType) {
		switch (iconType) {
		case CANVAS_OVERLAY_ICON:
			return getClass().getResource("icons/filewriter_48.png");
		case PALETTE_ICON_LARGE:
			return getClass().getResource("icons/filewriter_32.png");
		case PALETTE_ICON_SMALL:
			return getClass().getResource("icons/filewriter_16.png");
		}
		return null;
	}

	/***************************************************************************************
	 * The getter and setter methods provided by the Parameterizable object. *
	 * StreamBase Studio uses them to determine the name and type of each
	 * property * and obviously, to set and get the property values. *
	 ***************************************************************************************/
	// Adapter Properties
	/**
	 * 
	 * @param filename the filename
	 */
	public void setFilename(String filename) {
		this.filename = filename;
	}

	/**
	 * 
	 * @return the filename
	 */
	public String getFilename() {
		return this.filename;
	}

	/**
	 * 
	 * @return control filename
	 */
	public String getControlFilename() {
		return this.controlFilename;
	}

	/**
	 * 
	 * @param useDefaultCharset use default charset
	 */
	public void setUseDefaultCharset(boolean useDefaultCharset) {
		this.useDefaultCharset = useDefaultCharset;
	}

	/**
	 * 
	 * @return use default charset
	 */
	public boolean getUseDefaultCharset() {
		return this.useDefaultCharset;
	}

	/**
	 * 
	 * @param characterSet character set
	 */
	public void setCharacterSet(String characterSet) {
		this.characterSet = characterSet;
	}

	/**
	 * 
	 * @return character set
	 */
	public String getCharacterSet() {
		return this.characterSet;
	}

	/**
	 * 
	 * @param writeAsBinary read as binary
	 */
	public void setWriteAsBinary(boolean writeAsBinary) {
		this.writeAsBinary = writeAsBinary;
	}

	/**
	 * 
	 * @return write as binary
	 */
	public boolean getWriteAsBinary() {
		return this.writeAsBinary;
	}

	/**
	 * 
	 * @param synchronousWrite synchronous write
	 */
	public void setSynchronousWrite(boolean synchronousWrite) {
		this.synchronousWrite = synchronousWrite;
	}

	/**
	 * 
	 * @return synchronous write
	 */
	public boolean getSynchronousWrite() {
		return this.synchronousWrite;
	}

	/**
	 * 
	 * @param passthroughFields passthorugh fields
	 */
	public void setPassthroughFields(boolean passthroughFields) {
		this.passthroughFields = passthroughFields;
	}

	/**
	 * 
	 * @return passthorugh fields
	 */
	public boolean getPassthroughFields() {
		return this.passthroughFields;
	}

	/**
	 * 
	 * @param eventPort event port
	 */
	public void setEventPort(boolean eventPort) {
		this.eventPort = eventPort;

		int inports = inputPortsNoControl;
		if(this.getControlPort() == true)
			inports = inputPortsWithControl;
		int outports = outputPortsNoEvent;
		if(this.getEventPort() == true)
			outports = outputPortsWithEvent;
		setPortHints(inports, outports);
	}

	/**
	 * 
	 * @return event port
	 */
	public boolean getEventPort() {
		return this.eventPort;
	}

	/**
	 * 
	 * @param renderAs render as
	 */
	public void setRenderAs(String renderAs) {
		this.renderAs = renderAs;
	}

	/**
	 * 
	 * @return render as
	 */
	public String getRenderAs() {
		return this.renderAs;
	}

	// Line Options
	/**
	 * 
	 * @param oneLinePerTuple one line per tuple
	 */
	public void setOneLinePerTuple(boolean oneLinePerTuple) {
		this.oneLinePerTuple = oneLinePerTuple;
	}

	/**
	 * 
	 * @return one line per tuple
	 */
	public boolean getOneLinePerTuple() {
		return this.oneLinePerTuple;
	}

	/**
	 * 
	 * @param lineSeparator line separator
	 */
	public void setLineSeparator(String lineSeparator) {
		this.lineSeparator = lineSeparator;
	}

	/**
	 * 
	 * @return line separator
	 */
	public String getLineSeparator() {
		return this.lineSeparator;
	}

	/**
	 * 
	 * @param listTupleName list tuple name
	 */
	public void setListTupleName(String listTupleName) {
		this.listTupleName = listTupleName;
	}

	/**
	 * 
	 * @return list tuple name
	 */
	public String getListTupleName() {
		return this.listTupleName;
	}

	// CSV Options
	/**
	 * 
	 * @param fieldDelimiter field delimiter
	 */
	public void setFieldDelimiter(String fieldDelimiter) {
		this.fieldDelimiter = fieldDelimiter;
	}

	/**
	 * 
	 * @return field delimiter
	 */
	public String getFieldDelimiter() {
		return this.fieldDelimiter;
	}

	/**
	 * 
	 * @param nullAsEmpty null as empty
	 */
	public void setNullAsEmpty(boolean nullAsEmpty) {
		this.nullAsEmpty = nullAsEmpty;
	}

	/**
	 * 
	 * @return null as empty
	 */
	public boolean getNullAsEmpty() {
		return this.nullAsEmpty;
	}

	/**
	 * 
	 * @param nullValue null value
	 */
	public void setNullValue(String nullValue) {
		this.nullValue = nullValue;
	}

	/**
	 * 
	 * @return null value
	 */
	public String getNullValue() {
		return this.nullValue;
	}

	/**
	 * 
	 * @param timestampFormat timestamp format
	 */
	public void setTimestampFormat(String timestampFormat) {
		this.timestampFormat = timestampFormat;
	}

	/**
	 * 
	 * @return timestamp format
	 */
	public String getTimestampFormat() {
		return this.timestampFormat;
	}

	/**
	 * 
	 * @return file doesn't exist
	 */
	public String getFileDoesntExist() {
		return fileDoesntExist;
	}

	/**
	 * 
	 * @param fileDoesntExist file doesn't exist
	 */
	public void setFileDoesntExist(String fileDoesntExist) {
		this.fileDoesntExist = fileDoesntExist;
		if(this.fileDoesntExist.equals("Create new file"))
			this.fileDoesntExistAction = FileDoesntExist.CREATE_NEW_FILE;
		else if(this.fileDoesntExist.equals("Fail"))
			this.fileDoesntExistAction = FileDoesntExist.FAIL;
	}

	/**
	 * 
	 * @return file doesn't exist action
	 */
	public FileDoesntExist getFileDoesntExistAction() {
		return fileDoesntExistAction;
	}
	
	/**
	 * 
	 * @return file exists
	 */
	public String getFileExists() {
		return fileExists;
	}

	/**
	 * 
	 * @param fileExists file exists
	 */
	public void setFileExists(String fileExists) {
		this.fileExists = fileExists;
		if(this.fileExists.equals("Append to existing file"))
			this.fileExistsAction = FileExists.APPEND_TO_EXISTING;
		else if(this.fileExists.equals("Truncate existing file"))
			this.fileExistsAction = FileExists.TRUNCATE_EXISTING;
		else if(this.fileExists.equals("Fail"))
			this.fileExistsAction = FileExists.FAIL;
	}

	/**
	 * 
	 * @return file exists action
	 */
	public FileExists getFileExistsAction() {
		return this.fileExistsAction;
	}
	
	/**
	 * 
	 * @return control port
	 */
	public boolean getControlPort() {
		return controlPort;
	}

	/**
	 * 
	 * @param controlPort control port
	 */
	public void setControlPort(boolean controlPort) {
		this.controlPort = controlPort;

		int inports = inputPortsNoControl;
		if(this.getControlPort() == true)
			inports = inputPortsWithControl;
		int outports = outputPortsNoEvent;
		if(this.getEventPort() == true)
			outports = outputPortsWithEvent;
		setPortHints(inports, outports);
	}

	/**
	 * 
	 * @return include header
	 */
	public boolean getIncludeHeader() {
		return includeHeader;
	}

	/**
	 * 
	 * @param includeHeader include header
	 */
	public void setIncludeHeader(boolean includeHeader) {
		this.includeHeader = includeHeader;
	}

	/**
	 * For detailed information about shouldEnable methods, see interface
	 * Parameterizable java doc
	 * 
	 * @see Parameterizable
	 */

	// Adapter Properties
	/**
	 * 
	 * @return enable filename
	 */
	public boolean shouldEnableFilename() {
		return true;
	}

	/**
	 * 
	 * @return enable file doesn't exist
	 */
	public boolean shouldEnableFileDoesntExist() {
		return true;
	}

	/**
	 * 
	 * @return enable file exists
	 */
	public boolean shouldEnableFileExists() {
		return true;
	}

	/**
	 * 
	 * @return enable use default charset
	 */
	public boolean shouldEnableUseDefaultCharset() {
		if(this.getWriteAsBinary() == false)
			return true;
		return false;
	}

	/**
	 * 
	 * @return enable character set
	 */
	public boolean shouldEnableCharacterSet() {
		if (this.getUseDefaultCharset() == false && this.getWriteAsBinary() == false)
			return true;
		return false;
	}

	/**
	 * 
	 * @return enable write as binary
	 */
	public boolean shouldEnableWriteAsBinary() {
		if(this.getRenderAs().equals("None"))
			return true;
		return false;
	}

	/**
	 * 
	 * @return enable synchronous write
	 */
	public boolean shouldEnableSynchronousWrite() {
		return true;
	}

	/**
	 * 
	 * @return enable render as
	 */
	public boolean shouldEnableRenderAs() {
		if(this.getWriteAsBinary() == true)
			return false;
		return true;
	}

	/**
	 * 
	 * @return enable control port
	 */
	public boolean shouldEnableControlPort() {
		return true;
	}

	/**
	 * 
	 * @return enable event port
	 */
	public boolean shouldEnableEventPort() {
		return true;
	}

	/**
	 * 
	 * @return enable passthrough fields
	 */
	public boolean shouldEnablePassthroughFields() {
		return true;
	}

	// Line Options
	/**
	 * 
	 * @return enable one line per tuple
	 */
	public boolean shouldEnableOneLinePerTuple() {
		if (this.getRenderAs().equals("Line")
				|| this.getRenderAs().equals("CSV"))
			return true;
		return false;
	}

	/**
	 * 
	 * @return enable line separator
	 */
	public boolean shouldEnableLineSeparator() {
		if ((this.getRenderAs().equals("Line") || this.getRenderAs().equals(
				"CSV")))
			return true;
		return false;
	}

	/**
	 * 
	 * @return enable list tuple name
	 */
	public boolean shouldEnableListTupleName() {
		if ((this.getRenderAs().equals("Line") || this.getRenderAs().equals(
				"CSV"))
				&& this.getOneLinePerTuple() == false)
			return true;
		return false;
	}

	// CSV Options
	/**
	 * 
	 * @return enable include header
	 */
	public boolean shouldEnableIncludeHeader() {
		if (this.getRenderAs().equals("CSV"))
			return true;
		return false;
	}

	/**
	 * 
	 * @return enable field delimiter
	 */
	public boolean shouldEnableFieldDelimiter() {
		if (this.getRenderAs().equals("CSV"))
			return true;
		return false;
	}

	/**
	 * 
	 * @return enable null as empty
	 */
	public boolean shouldEnableNullAsEmpty() {
		if (this.getRenderAs().equals("CSV"))
			return true;
		return false;
	}

	/**
	 * 
	 * @return enable null value
	 */
	public boolean shouldEnableNullValue() {
		if (this.getRenderAs().equals("CSV") && this.getNullAsEmpty() == false)
			return true;
		return false;
	}

	/**
	 * 
	 * @return enable timestamp format
	 */
	public boolean shouldEnableTimestampFormat() {
		if (this.getRenderAs().equals("CSV"))
			return true;
		return false;
	}

	/**
	 * 
	 * @return enable input object schema
	 */
	public Schema getInputObjectSchema() {
		return this.inputObjectSchema;
	}
}
