/*
* Copyright © 2020. TIBCO Software Inc.
* This file is subject to the license terms contained
* in the license file that is distributed with this file.
*/

package com.tibco.accelerators.operators.file.handler;

import java.io.File;

import org.slf4j.Logger;

import com.streambase.sb.Schema;
import com.streambase.sb.StreamBaseException;
import com.streambase.sb.Tuple;
import com.streambase.sb.TupleException;
import com.tibco.accelerators.operators.file.FileDeleteOperator;

public class FileDeleteHandler extends Thread{
	private FileDeleteOperator operator = null;
	private Logger logger = null;
	private Tuple inputTuple = null;

	public FileDeleteHandler(Logger logger, FileDeleteOperator operator, Tuple tuple) {
		this.logger = logger;
		this.operator = operator;
		this.inputTuple = tuple;
	}
	
	public void run() {
		// Setup the filenames
		String filename = this.getFilename();		
		
		// Create output
		Schema dataOutputSchema = this.operator.getCachedOutputSchema(0);
		Tuple dataOutputTuple = dataOutputSchema.createTuple();

		try {
			// Validate the file exists
			this.validateFileExists(filename);
			
			// Delete file
			File file = new File(filename);			
			file.delete();
			
			dataOutputTuple.setField("filename", filename);
			dataOutputTuple.setField("result", "Success");
			this.sendDataTuple(dataOutputTuple);
		} catch (Exception ex) {
			//ex.printStackTrace();
			this.logger.error(ex.getMessage());
			try {
				dataOutputTuple.setField("filename", filename);
				dataOutputTuple.setField("result", ex.getMessage());
				this.sendDataTuple(dataOutputTuple);
			}
			catch(Exception e) {
				e.printStackTrace();
				this.logger.error(e.getMessage());
			}
		}
	}

	private String getFilename() {
		// Setup the filename
		String filename = null;
		
		try {
			filename = this.inputTuple.getString("filename");
		} catch (TupleException e1) {
			filename = this.operator.getFilename();
		}
		
		return filename;
	}
		
	private void validateFileExists(String filename) throws StreamBaseException {
		if(filename == null || new File(filename).exists() == false) {
			String message = "Error opening file '" + filename + "' not found";
			throw new StreamBaseException(message);
		}
	}
	
	private void sendDataTuple(Tuple dataOutputTuple) throws StreamBaseException {
		// Append passthrough
		this.appendPassthrough(dataOutputTuple);
		
		// Send data tuple
		this.operator.sendOutput(0, dataOutputTuple);
	}
		
	private void appendPassthrough(Tuple dataOutputTuple) throws TupleException {
		if(this.operator.getPassthroughFields() == true) {
			dataOutputTuple.setField("passthrough", this.inputTuple.getField("passthrough"));
		}
	}
}
