/*
* Copyright © 2020. TIBCO Software Inc.
* This file is subject to the license terms contained
* in the license file that is distributed with this file.
*/

package com.tibco.accelerators.operators.file.handler;

import java.io.File;

import org.slf4j.Logger;

import com.streambase.sb.Schema;
import com.streambase.sb.StreamBaseException;
import com.streambase.sb.Tuple;
import com.streambase.sb.TupleException;
import com.tibco.accelerators.operators.file.FileMoveOperator;

public class FileMoveHandler extends Thread{
	private FileMoveOperator operator = null;
	private Logger logger = null;
	private Tuple inputTuple = null;

	public FileMoveHandler(Logger logger, FileMoveOperator operator, Tuple tuple) {
		this.logger = logger;
		this.operator = operator;
		this.inputTuple = tuple;
	}
	
	public void run() {
		// Setup the filenames
		String sourceFilename = this.getSourceFilename();		
		String targetFilename = this.getTargetFilename();
		
		Schema dataOutputSchema = this.operator.getCachedOutputSchema(0);
		Tuple dataOutputTuple = dataOutputSchema.createTuple();
		try {
			// Validate the source file exists
			this.validateSourceFileExists(sourceFilename);
			
			// Check if target file exists
			this.checkTargetFileExists(targetFilename);
			
			// Move file
			File sourceFile = new File(sourceFilename);
			File targetFile = new File(targetFilename);
			
			if(targetFile.exists() == true)
				targetFile.delete();
			
			sourceFile.renameTo(targetFile);
			
			dataOutputTuple.setField("sourceFilename", sourceFilename);
			dataOutputTuple.setField("targetFilename", targetFilename);
			dataOutputTuple.setField("result", "Success");
			this.sendDataTuple(dataOutputTuple);
		} catch (Exception ex) {
			//ex.printStackTrace();
			this.logger.error(ex.getMessage());
			try {
				dataOutputTuple.setField("sourceFilename", sourceFilename);
				dataOutputTuple.setField("targetFilename", targetFilename);
				dataOutputTuple.setField("result", ex.getMessage());
				this.sendDataTuple(dataOutputTuple);
			}
			catch(Exception e) {
				e.printStackTrace();
				this.logger.error(e.getMessage());
			}
		}
	}

	private String getSourceFilename() {
		// Setup the filename
		String filename = null;
		
		try {
			filename = this.inputTuple.getString("sourceFilename");
		} catch (TupleException e1) {
			filename = this.operator.getSourceFilename();
		}
		
		return filename;
	}
	
	private String getTargetFilename() {
		// Setup the filename
		String filename = null;
		
		try {
			filename = this.inputTuple.getString("targetFilename");
		} catch (TupleException e1) {
			filename = this.operator.getTargetFilename();
		}
		
		return filename;
	}
	
	private void validateSourceFileExists(String filename) throws StreamBaseException {
		if(filename == null || new File(filename).exists() == false) {
			String message = "Error opening source file '" + filename + "' not found";
			throw new StreamBaseException(message);
		}
	}
	
	private void checkTargetFileExists(String filename) throws StreamBaseException {
		if(this.operator.getOverwriteExisting() == false && filename != null && new File(filename).exists() == true) {
			String message = "Error writing target file '" + filename + "' already exists";
			throw new StreamBaseException(message);
		}
	}
	
	private void sendDataTuple(Tuple dataOutputTuple) throws StreamBaseException {
		// Append passthrough
		this.appendPassthrough(dataOutputTuple);
		
		// Send data tuple
		this.operator.sendOutput(0, dataOutputTuple);
	}
		
	private void appendPassthrough(Tuple dataOutputTuple) throws TupleException {
		if(this.operator.getPassthroughFields() == true) {
			dataOutputTuple.setField("passthrough", this.inputTuple.getField("passthrough"));
		}
	}
}
