/*
* Copyright © 2020. TIBCO Software Inc.
* This file is subject to the license terms contained
* in the license file that is distributed with this file.
*/
package com.tibco.accelerators.adapter.http;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringWriter;
import java.net.HttpURLConnection;
import java.net.URL;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class RelayServlet extends HttpServlet {

	private static final long serialVersionUID = 1431086787480169906L;
	
	@Override
	protected void doGet(HttpServletRequest req, HttpServletResponse res)
			throws ServletException, IOException {

		String server = getInitParameter("TargetUrl");
		String remoteUrl = req.getPathInfo();
		String queryString = req.getQueryString();
		String targetUrl = server + remoteUrl + "?" + queryString;

		this.execute(targetUrl, "GET", res);
	}

	@Override
	protected void doPost(HttpServletRequest req, HttpServletResponse res)
			throws ServletException, IOException {
		
		String server = getInitParameter("TargetUrl");
		String remoteUrl = req.getPathInfo();
		String postData = this.getPostData(req);
		String targetUrl = server + remoteUrl;
		
		this.execute(targetUrl, "POST", postData, req.getContentType(), res);
	}

	@Override
	protected void doPut(HttpServletRequest req, HttpServletResponse res)
			throws ServletException, IOException {

		String server = getInitParameter("TargetUrl");
		String remoteUrl = req.getPathInfo();
		String postData = this.getPostData(req);
		String targetUrl = server + remoteUrl;
		
		this.execute(targetUrl, "PUT", postData, req.getContentType(), res);
	}

	@Override
	protected void doDelete(HttpServletRequest req, HttpServletResponse res) {

		String server = getInitParameter("TargetUrl");
		String remoteUrl = req.getPathInfo();
		String targetUrl = server + remoteUrl;
		
		this.execute(targetUrl, "DELETE", res);
	}
	
	private String getPostData(HttpServletRequest req) throws ServletException {
		try {
			StringWriter writer = new StringWriter();
			Reader reader = req.getReader();
			char[] buffer = new char[8192];
			for (;;) {
				int r = reader.read(buffer);
				if (r < 0)
					break;
				writer.write(buffer, 0, r);
			}

			return writer.toString();
		} catch (Exception e) {
			throw new ServletException(e);
		}
	}
	
	private void execute(String targetUrl, String method, HttpServletResponse res) {
		this.execute(targetUrl, method, null, null, res);
	}
	
	private void execute(String targetUrl, String method, String postData, String contentType, HttpServletResponse res) {
		URL url;
		HttpURLConnection connection = null;
		try {
			// Create connection
			url = new URL(targetUrl);
			connection = (HttpURLConnection) url.openConnection();
			connection.setRequestMethod(method);
			connection.setUseCaches(false);
			connection.setDoInput(true);
			
			if(postData != null) {
				connection.setRequestProperty("Content-Type", contentType);
				connection.setRequestProperty("Content-Length",
						"" + Integer.toString(postData.getBytes().length));
				connection.setDoOutput(true);

				DataOutputStream wr = new DataOutputStream(
						connection.getOutputStream());
				wr.writeBytes(postData);			
				wr.flush();
				wr.close();
			}

			// Handle response
			String responseStr;
			switch(connection.getResponseCode()) {
				case 200:
					responseStr = handleSuccess(connection);
					break;
					
				default:
					responseStr = handleError(connection);
			}
			
			// Relay response back to origin
			res.setStatus(connection.getResponseCode());
			res.setContentType(connection.getContentType());
			res.setContentLengthLong(connection.getContentLengthLong());
			res.getWriter().write(responseStr);			
			
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if (connection != null) {
				connection.disconnect();
			}
		}		
	}
	
	private String handleSuccess(HttpURLConnection connection) throws IOException {
		InputStream is = connection.getInputStream();
		return handleStream(is);
	}
	
	private String handleError(HttpURLConnection connection) throws IOException {
		InputStream is = connection.getErrorStream();
		return handleStream(is);
	}
	
	private String handleStream(InputStream is) throws IOException {
		BufferedReader rd = new BufferedReader(new InputStreamReader(is));
		String line;
		StringBuffer response = new StringBuffer();
		while ((line = rd.readLine()) != null) {
			response.append(line);
			response.append('\r');
		}
		rd.close();
		
		return response.toString();
	}
	
	
}
